#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>
#include <string.h>

#include "logger.h"
Logger logger;

void create_logger(char* logpath, LogLevel level) {
    logger = malloc(sizeof(*logger));

    logger->t0 = clock();
    logger->level = level;
    
    if (logpath[0] != '\0')
    {
        logger->logfile = fopen(logpath, "w");
    }
    else
        logger->logfile = NULL;
}

void destroy_logger() {
    if (logger->logfile != NULL)
        fclose(logger->logfile);
    free(logger);
}

void logprint(LogLevel level, char* str, va_list args)
{
    clock_t t = clock();

    char label[6];
    switch (level)
    {
        case (LOG_PANIC):
            strcpy(label, "PANIC");
            break;
        case (LOG_FATAL):
            strcpy(label, "FATAL");
            break;
        case (LOG_ERROR):
            strcpy(label, "ERROR");
            break;
        case (LOG_WARN):
            strcpy(label, "WARN");
            break;
        case (LOG_INFO):
            strcpy(label, "INFO");
            break;
        case (LOG_DEBUG):
            strcpy(label, "DEBUG");
            break;
        case (LOG_TRACE):
            strcpy(label, "TRACE");
            break;
    }

    FILE* stream = (level >= LOG_WARN) ? stderr : stdout;

    if (level < logger->level)
        return;

    fprintf(stream, "[%s][%f]: ", label, (double)t / CLOCKS_PER_SEC);
    vfprintf(stream, str, args);
    fprintf(stream, "\n");

    if (logger->logfile != NULL)
    {
        fprintf(logger->logfile, "[%s][%f]: ", label, (double)t / CLOCKS_PER_SEC);
        vfprintf(logger->logfile, str, args);
        fprintf(logger->logfile, "\n");
    }
}


void panic(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_PANIC, str, args);
    va_end(args);
}

void fatal(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_FATAL, str, args);
    va_end(args);
}

void error(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_ERROR, str, args);
    va_end(args);
}

void warn(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_WARN, str, args);
    va_end(args);
}

void info(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_INFO, str, args);
    va_end(args);
}

void debug(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_DEBUG, str, args);
    va_end(args);
}

void trace(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_TRACE, str, args);
    va_end(args);
}
