/*
 *   Colloscope - A program that generates a colloscope for French 'classes prépas'
 *  Copyright (C) 2024  Alexandre Aboujaib
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

#include "logger.h"
#include "structure.h"
#include "algorithm.h"

// gcc -g -Wall -Wextra -Wpedantic main.c -lSDL2 -lSDL2_image -lm -o main
// ./main MP2I-creneaux.txt 33 MP2I-colleurs.txt 16 6 15 output.csv

int main(int argc, char **argv) {
    if (argc!=8) {
        fprintf(stderr, "Usage: %s <creneaux> <n_creneaux> <colleurs> <n_colleurs> <n_weeks> <n_groups> <output>\n", argv[0]);
        exit(1);
    }
    
    create_logger("colloscope.log", LOG_TRACE);

    char* path_creneaux = argv[1];
    int n_creneaux = str_to_int(argv[2]);
    char* path_colleurs = argv[3];
    int n_colleurs = str_to_int(argv[6]);
    int n_weeks = str_to_int(argv[5]);
    int n_groups = str_to_int(argv[6]);
    char* path_output = argv[7];

    trace("%d %d %d %d", n_creneaux, n_colleurs, n_weeks, n_groups);

    info("Starting");
    srand(time(NULL));

    //creneau* edt = import_creneaux("file.txt", 76);
    creneau* edt = import_creneaux_oneweek(path_creneaux, n_creneaux, n_weeks);
    int len_edt = n_creneaux*n_weeks;

    //colleur* dudes = import_colleurs("some_data.txt", 13, len_creneau);
    colleur* dudes = import_colleurs_oneweek(path_colleurs, 16, 6, 33);

    aux_2(edt, len_edt, dudes, n_colleurs, n_groups, n_weeks, 5000, path_output);

    for(int i = 0; i < len_edt; i++) {
        free(edt[i].name);
    }
    free(edt);

    for(int i = 0; i < n_colleurs; i++) {
        free(dudes[i].disp);
        free(dudes[i].name);
    }
    free(dudes);

    destroy_logger();

    return 0;
}
